import std;

//Most common WinAPI functions, structures and macros are contained within:
#include <windows.h>

//Another often-used header is windowsx.h

//Desktop programs on Windows have a different entry function compared to standard C or C++, see:
//https://learn.microsoft.com/en-us/windows/win32/learnwin32/winmain--the-application-entry-point
int WINAPI wWinMain(HINSTANCE /* instance - instance handle of loaded program executable, which can be used
								 e.g. to access resources (icons, bitmaps, strings, other files) embedded
								 in the .exe file */,
					HINSTANCE /* prev_instance - always NULL, no meaning nowadays */,
					PWSTR /* cmd_line - program's command line arguments, not split like in main in C/C++,
							 If you want them split, use CommandLineToArgvW from shellapi.h:
							 https://learn.microsoft.com/en-us/windows/win32/api/shellapi/nf-shellapi-commandlinetoargvw
						  */,
					int /* show_command - recommended show state of the main window */)
{
	//see https://learn.microsoft.com/en-us/windows/win32/api/winuser/nf-winuser-messageboxw for more information
	int result = MessageBoxW(nullptr /* parent window [optional] - message box is a modal window which means it blocks
									    interaction with the parent untill it is dismissed */,
							 L"Hello WinAPI!" /* text of the message box */,
							 L"My First WinAPI Program" /* title of the message box window */,
							 MB_OKCANCEL /* flags controlling the message box, e.g. which buttons are shown,
										    which icon is used, etc. */);

	//The return value can be used to determine which message box button was clicked
	std::wstring output = std::format(L"User clicked {}!\n", result == IDOK ? L"OK" : L"Cancel");

	//Prints a debug message, which can be seen in the Output window, for more information see:
	//https://learn.microsoft.com/en-us/windows/win32/api/debugapi/nf-debugapi-outputdebugstringw
	OutputDebugStringW(output.c_str());
	return 0;
}