module;

#include <windows.h>
#include <cassert>

export module pige.winapi:exceptions;

import std;

export namespace pige::winapi
{
	class winapi_error
	{
		DWORD m_error_code;
		std::stacktrace m_trace;

	public:
		inline explicit winapi_error(DWORD error_code = GetLastError(),
									 std::stacktrace trace = std::stacktrace::current()) noexcept
			: m_error_code{ error_code }, m_trace{ std::move(trace) }
		{ }

		[[nodiscard]] std::wstring what() const noexcept
		{
			std::wstring message;
			LPWSTR buffer = nullptr;
			try
			{
				auto trace_str = to_string(m_trace);
				std::wstring trace_wstr;
				assert(trace_str.size() <= std::numeric_limits<int>::max());
				auto converted_length = MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, trace_str.data(),
															static_cast<int>(trace_str.size()), nullptr, 0);
				trace_wstr.resize(converted_length);
				MultiByteToWideChar(CP_UTF8, MB_PRECOMPOSED, trace_str.data(), static_cast<int>(trace_str.size()),
					trace_wstr.data(), converted_length);
				if (FormatMessageW(
						FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS,
						nullptr, m_error_code, MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
						reinterpret_cast<LPWSTR>(&buffer), 0, nullptr))
					message = std::format(L"Error: {}in:\n{}", buffer, trace_wstr);
				else
					message = std::format(L"Error code {} in\n{}", m_error_code, trace_wstr);
			}
			catch (...) { }
			LocalFree(buffer);
			return message;
		}
	};
}